#!/usr/bin/python

import sys
sys.path.insert(0,"../")

import logging
import unittest

from softwarecenter.db.pkginfo import get_pkg_info, _Package, _Version

class TestPkgInfo(unittest.TestCase):

    def setUp(self):
        pass

    def test_pkg_version(self):
        pkginfo = get_pkg_info()
        pkginfo.open()

        pkg = pkginfo['coreutils']
        self.assertTrue(isinstance(pkg, _Package))
        self.assertTrue(pkg.is_installed)
        self.assertTrue(isinstance(pkg.installed, _Version))
        self.assertTrue(isinstance(pkg.candidate, _Version))

        self.assertNotEqual(len(pkg.installed.origins), 0)
        self.assertNotEqual(len(pkg.installed.summary), '')
        self.assertNotEqual(len(pkg.installed.description), '')
        self.assertNotEqual(pkg.installed.size, 0)
        self.assertNotEqual(pkg.installed.installed_size, 0)

        for v in pkg.versions:
            self.assertTrue(isinstance(v, _Version))

    def test_pkg_info(self):
        pkginfo = get_pkg_info()
        pkginfo.open()
        self.assertTrue(pkginfo.is_installed("coreutils"))
        self.assertTrue(pkginfo.is_available("bash"))
        self.assertTrue(len(pkginfo.get_addons("firefox")) > 0)
        self.assertEqual(pkginfo.get_section('bash'), 'shells')
        self.assertTrue('GNU Bourne Again' in pkginfo.get_summary('bash'))
        self.assertTrue(pkginfo.get_description('bash') != '')
        self.assertTrue(len(pkginfo.get_origins("firefox")) > 0)
        self.assertTrue(pkginfo.get_installed("coreutils") is not None)
        self.assertTrue(pkginfo.get_candidate("coreutils") is not None)
        self.assertTrue(len(pkginfo.get_versions("coreutils")) != 0)

        self.assertTrue('coreutils' in pkginfo)
        
        pkg = pkginfo['coreutils']
        self.assertTrue(len(pkginfo.get_packages_removed_on_remove(pkg)) != 0)
        self.assertTrue(len(pkginfo.get_packages_removed_on_install(pkg)) == 0)
        self.assertTrue(pkg is not None)
        self.assertTrue(pkg.is_installed)
        self.assertTrue(len(pkg.versions) != 0)
        self.assertEqual(pkg.section, "utils")
        self.assertEqual(pkg.website, 'http://gnu.org/software/coreutils')

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()
